@@ -2,31 +2,39 @@ package Excel::ValueReader::XLSX;
 use 5.12.1;
 use utf8;
 use Moose;
-use Module::Load          qw/load/;
-use Date::Calc            qw/Add_Delta_Days/;
-use POSIX                 qw/strftime modf/;
-use Carp                  qw/croak/;
-
-our $VERSION = '1.14';
+use MooseX::StrictConstructor;
+use Moose::Util::TypeConstraints qw/union/;
+use Module::Load                 qw/load/;
+use Date::Calc                   qw/Add_Delta_Days/;
+use POSIX                        qw/strftime modf/;
+use Carp                         qw/croak/;
+use Iterator::Simple             qw/iter/;
 
 #======================================================================
-# ATTRIBUTES
+# GLOBALS
 #======================================================================
 
+our $VERSION = '1.16';
+our %A1_to_num_memoized;
+
+#======================================================================
+# TYPES AND ATTRIBUTES
+#======================================================================
+
+# TYPES
+my $XlsxSource = union([qw/Str FileHandle/]);
+
 # PUBLIC ATTRIBUTES
-has 'xlsx'            => (is => 'ro', isa => 'Str|FileHandle', required => 1);      # path of xlsx file
-has 'using'           => (is => 'ro', isa => 'Str', default => 'Regex'); # name of backend class
-has 'date_format'     => (is => 'ro', isa => 'Str', default => '%d.%m.%Y');
-has 'time_format'     => (is => 'ro', isa => 'Str', default => '%H:%M:%S');
+has 'xlsx'            => (is => 'ro', isa => $XlsxSource,      required => 1);      # path of xlsx file
+has 'using'           => (is => 'ro', isa => 'Str',            default => 'Regex'); # name of backend class
+has 'date_format'     => (is => 'ro', isa => 'Str',            default => '%d.%m.%Y');
+has 'time_format'     => (is => 'ro', isa => 'Str',            default => '%H:%M:%S');
 has 'datetime_format' => (is => 'ro', isa => 'Str',            builder => '_datetime_format', lazy => 1);
-has 'date_formatter'  => (is => 'ro', isa => 'Maybe[CodeRef]', builder => '_date_formatter', lazy => 1);
-
-
+has 'date_formatter'  => (is => 'ro', isa => 'Maybe[CodeRef]', builder => '_date_formatter',  lazy => 1);
 
 # ATTRIBUTES USED INTERNALLY, NOT DOCUMENTED
-has 'backend'         => (is => 'ro',   isa => 'Object', init_arg => undef,
-                          builder => '_backend', lazy => 1,
-                          handles => [qw/values base_year sheets active_sheet/]);
+has 'backend'         => (is => 'ro', isa => 'Object',         builder => '_backend',         lazy => 1,
+                          init_arg => undef, handles => [qw/base_year sheets active_sheet table_info/]);
 
 #======================================================================
 # BUILDING
@@ -37,17 +45,16 @@ around BUILDARGS => sub {
   my $orig  = shift;
   my $class = shift;
 
-  return ( @_ == 1 && !ref $_[0] ) ? $class->$orig(xlsx => $_[0])
-                                   : $class->$orig(@_);
-};
+  unshift @_, 'xlsx' if scalar(@_) % 2 and $XlsxSource->check($_[0]);
 
+  $class->$orig(@_);
+};
 
 
 #======================================================================
 # ATTRIBUTE CONSTRUCTORS
 #======================================================================
 
-
 sub _backend {
   my $self = shift;
 
@@ -99,28 +106,15 @@ sub _date_formatter {
 sub sheet_names {
   my ($self) = @_;
 
-  my $sheets = $self->sheets; # arrayref of shape {$name => $sheet_position}
+  my $sheets = $self->sheets; # hashref of shape {$name => $sheet_position}
 
   my @sorted_names = sort {$sheets->{$a} <=> $sheets->{$b}} keys %$sheets;
   return @sorted_names;
 }
 
 
-sub A1_to_num { # convert Excel A1 reference format to a number
-  my ($self, $string) = @_;
-
-  # ordinal number for character just before 'A'
-  state $base = ord('A') - 1;
-
-  # iterate on 'digits' (letters of the A1 cell reference)
-  my $num = 0;
-  foreach my $digit (map {ord($_) - $base} split //, $string) {
-    $num = $num*26 + $digit;
-  }
-
-  return $num;
-}
-
+sub values  { my ($self, $sheet) = @_; $self->backend->_values($sheet, 0)}
+sub ivalues { my ($self, $sheet) = @_; $self->backend->_values($sheet, 1)}
 
 sub formatted_date {
   my ($self, $val, $date_format, $date_formatter) = @_;
@@ -178,90 +172,125 @@ sub formatted_date {
 sub table_names {
   my ($self) = @_;
 
-  my $table_info = $self->backend->table_info;
+  my $table_info = $self->table_info;
 
   # sort on table id (field [1] in table_info arrayrefs)
-  my @table_names = sort {$table_info->{$a}[1] <=> $table_info->{$b}->[1]} keys %$table_info;
+  my @table_names = sort {$table_info->{$a}{id} <=> $table_info->{$b}{id}} keys %$table_info;
 
   return @table_names;
 }
 
+sub table  {shift->_table(0, @_);} # 0 = does not want an iterator, just a regular arrayref
+sub itable {shift->_table(1, @_);} # 1 = does want an iterator
 
-# info fields returned from the backend parsing methods
-my @table_info_fields = qw/sheet table_id ref columns no_headers/;
-
-
-# the same fields are also the valid args for the method call
-my $is_valid_arg      = "^(" . join("|", @table_info_fields) . ")\$";
-
-sub table {
-  my $self = shift;
+my %valid_table_arg = map {$_ => 1} qw/name sheet ref columns no_headers with_totals want_records/;
+sub _table {
+  my $self          = shift;
+  my $want_iterator = shift;
 
   # syntactic sugar : ->table('foo') is treated as ->table(name => 'foo')
-  my %args = @_ == 1 ? (name => $_[0]) : @_;
+  unshift @_, 'name' if scalar(@_) % 2;
+  my %args = @_;
 
-  # if called with a table name, derive all other args from internal workbook info
+  # check for invalid args
+  my @invalid_args = grep {!$valid_table_arg{$_}} keys %args;
+  croak "invalid table args: ", join(", ", @invalid_args) if @invalid_args;
+
+  # defaults
+  $args{want_records} //= 1;
+
+  # if called with a table name, derive positional args from the internal workbook info
   if (my $table_name = delete $args{name}) {
-    !$args{$_} or croak "table() : arg '$_' is incompatible with 'name'"  for @table_info_fields;
-    my $table_info = $self->backend->table_info->{$table_name}
+    my $table_info = $self->table_info->{$table_name}
       or croak sprintf "Excel file '%s' contains no table named '%s'", $self->xlsx, $table_name;
-    @args{@table_info_fields} = @$table_info;
+    $args{$_} //= $table_info->{$_} for keys %$table_info;
   }
 
-  # check args
-  my @invalid_args = grep {!/$is_valid_arg/} keys %args;
-  croak "invalid args to table(): " . join ", ", @invalid_args if @invalid_args;
+  # get values from the sheet
+  my ($sheet_ref, $vals_or_it) = $want_iterator ? $self->ivalues($args{sheet})
+                                                : $self->values($args{sheet});
 
-  # get raw values from the sheet
-  my $values = $self->values($args{sheet});
+  # table boundaries 
+  my ($scol1, $srow1, $scol2, $srow2) = $self->range_from_ref($sheet_ref);
+  my ($tcol1, $trow1, $tcol2, $trow2) = $self->range_from_ref($args{ref});
+  my $skip_initial_rows               = $trow1 - $srow1;
+  my $keep_rows                       = $trow2 - $trow1 + 1;
+  my $skip_initial_cols               = $tcol1 - $scol1;
+  my $keep_cols                       = $tcol2 - $tcol1 + 1;
 
-  # restrict values to the table subrange (if applicable)
-  $values = $self->_subrange($values, $args{ref}) if $args{ref};
+  # if a totals row is present, skip it, unless the 'with_totals' arg is present
+  $keep_rows -=1 if $args{has_totals} and !$args{with_totals};
 
-  # take headers from first row if not already given in $args{columns}
-  $args{columns} //= $values->[0];
+  # skip initial rows if the table does not start at top row
+  if ($skip_initial_rows) {
+    if ($want_iterator) {$vals_or_it->() while $skip_initial_rows-- > 0;}
+    else                {splice @$vals_or_it, 0, $skip_initial_rows}
+  }
 
-  # if this table has headers (which is almost always the case), drop the header row
-  shift @$values unless $args{no_headers};
+  # read headers from first row -- even if this may be redundant with the 'columns' list declared in the table description
+  my $headers;
+  unless ($args{no_headers}) {
+    $keep_rows--; 
+    $headers = $want_iterator ? $vals_or_it->() : shift @$vals_or_it;
+    splice @$headers, 0, $skip_initial_cols if $skip_initial_cols;
+    splice @$headers, $keep_cols;
+  }
+  $args{columns}      //= $headers;
+  croak "table contains undefined columns" if grep {!defined $_} @{$args{columns}};
 
-  # build a table of hashes. This could be done with a simple map(), but using a loop
-  # avoids to store 2 copies of cell values in memory : @$values is shifted when @table is pushed.
-  my @cols = @{$args{columns}};
-  croak "table contains undefined columns" if grep {!defined $_} @cols;
+  # dual closure : can be used as an iterator or can compute all values in one call
   my @rows;
-  while (my $vals = shift @$values) {
-    my %row;
-    @row{@cols} = @$vals;
-    push @rows, \%row;
+  my $get_values = sub {
+    while (1) {
+      $keep_rows--
+        and my $vals = $want_iterator ? $vals_or_it->() : shift @$vals_or_it
+        or  return;                                                           # no more records -- end of iterator
+      splice @$vals, 0, $skip_initial_cols if $skip_initial_cols;
+      splice @$vals, $keep_cols;
+      my $row = $args{want_records} ? do {my %r; @r{@{$args{columns}}} = @$vals; \%r}
+                                    : $vals;
+      if ($want_iterator) {return $row}                                       # current iteration successful
+      else                {push @rows, $row};
+    }
+  };
+
+  # return either an iterator or the accumulated table records
+  return ($args{columns}, $want_iterator ? iter($get_values)
+                                         : do {$get_values->(); \@rows});
+}
+
+
+sub A1_to_num { # convert Excel A1 reference format to a number
+  my ($self, $A1) = @_;
+  my $num = 0;
+  foreach my $digit (unpack "C*", $A1) {
+    $num = $num*26 + $digit-64;
   }
 
-  # in scalar context, just return the rows. In list context, also return the column names
-  return wantarray ? (\@cols, \@rows) : \@rows;
+  return $num;
+}
+
+
+sub range_from_ref { # convert a range reference like 'C9:E21' into ($col1, $row1, $col2, $row2)
+  my ($self, $range) = @_;
+
+  $range =~ /^([A-Z]+)(\d+)                       # mandatory 1st col and row
+              (?:                                  # .. optionally followed by
+                 :                                 # colon
+                 ([A-Z]+)(\d+)                     # and 2nd col and row
+               )?
+              $/x
+     or croak "->range_from_ref($range) : invalid ref";
+
+  my @range        = ($A1_to_num_memoized{$1} //= $self->A1_to_num($1), $2); # col, row of topleft cell
+  push @range, ($3 ? ($A1_to_num_memoized{$3} //= $self->A1_to_num($3), $4)  # col, row of bottomright cell, or ..
+                   : @range);                                                # .. copy of topleft cell
+  return @range;
 }
 
 
 
-sub _subrange {
-  my ($self, $values, $ref) = @_;
 
-  # parse rows and columns from the $ref string (of shape like for example "A1:D34")
-  my ($col1, $row1, $col2, $row2) = $ref =~ /^([A-Z]+)(\d+):([A-Z]+)(\d+)$/
-    or croak "_subrange : invalid ref: $ref";
-
-  # restrict to the row range
-  if ($row1 > 1 || $row2 < @$values){
-    $values = [ @$values[$row1-1 .. $row2-1] ];
-  }
-
-  # restrict to the column range
-  my @col_nums = map {$self->A1_to_num($_) - 1} ($col1, $col2);
-  if ($col_nums[0] > 1){
-    my @col_range = ($col_nums[0] .. $col_nums[1]);
-    $values = [map { [ @$_[@col_range] ]} @$values];
-  }
-
-  return $values;
-}
 
 
 1;
@@ -291,24 +320,47 @@ Excel::ValueReader::XLSX - extracting values from Excel workbooks in XLSX format
   }
   
   foreach my $table_name ($reader->table_names) {
-     my ($columns, $rows) = $reader->table($table_name);
-     my $n_data_rows      = @$rows;
-     my $n_columns        = @$columns;
-     print "table $table_name has $n_data_rows rows and $n_columns columns; ",
-           "column 'foo' in first row contains : ", $rows->[0]{foo};
+     my ($columns, $records) = $reader->table($table_name);
+     my $n_records           = @$records;
+     my $n_columns           = @$columns;
+     print "table $table_name has $n_records records and $n_columns columns; ",
+           "column 'foo' in first row contains : ", $records->[0]{foo};
   }
   
-  my $first_grid = $reader->values(1); # if using numerical indices, start at 1
+  my $first_grid = $reader->values(1); # the arg can be a sheet index instead of a sheet name
+  
+  # iterator version of ->values()
+  my $iterator = $reader->ivalues($sheet_name);
+  while (my $row = $iterator->()) { process_row($row) }
+
+  # iterator version of ->table()
+  my ($columns, $iterator) = $reader->itable($table_name);
+  while (my $record = $iterator->()) { process_record($record) }
 
 =head1 DESCRIPTION
 
 =head2 Purpose
 
 This module reads the contents of an Excel file in XLSX format.
-Unlike other modules like L<Spreadsheet::ParseXLSX> or L<Spreadsheet::XLSX>, 
-there is no support for reading formulas, formats or other Excel internal
-information; all you get are plain values -- but you get them much
-faster ! Besides, this module also has support for parsing Excel tables.
+Unlike other modules like L<Spreadsheet::ParseXLSX> or L<Data::XLSX::Parser>, 
+this module has no support for reading formulas, formats or other Excel internal
+information; all you get are plain values -- but you get them much faster ! 
+
+Besides, this API has some features not found in concurrent parsers :
+
+=over
+
+=item *
+
+has support for parsing Excel tables
+
+=item *
+
+iterator methods for getting one row at a time from a worksheet or from a table -- very useful for sparing
+memory when dealing with large Excel files.
+
+=back
+
 
 =head2 Backends
 
@@ -318,12 +370,12 @@ Two different backends may be used for extracting values :
 
 =item Regex
 
-this backend uses regular expressions to parse the XML content.
+using regular expressions to parse the XML content.
 
 =item LibXML
 
-this backend uses L<XML::LibXML::Reader> to parse the XML content.
-It is probably safer but about three times slower than the Regex backend
+using L<XML::LibXML::Reader> to parse the XML content.
+It is probably safer but two to three times slower than the Regex backend
 (but still much faster than L<Spreadsheet::ParseXLSX>).
 
 =back
@@ -339,25 +391,64 @@ Here index 1 was chosen to be consistent with the common API for "collections" i
 Microsoft Office object model.
 
 
+=head1 NOTE ON ITERATORS
+
+Methods L</ivalues> and L</itable> return I<iterators>.
+Each call to the iterator produces a new data row from the Excel content, until reaching
+the end of data where the iterator returns C<undef>. Following the L<Iterator::Simple> protocol,
+iterators support three different but semantically equivalent syntaxes :
+
+  while (my $row = $iterator->())   { process($row) }
+  
+  while (my $row = $iterator->next) { process($row) }
+  
+  while (<$iterator>)               { process($_) }
+
+Working with iterators is especially interesting when dealing with large Excel files, because
+rows can be processed one at a time instead of being loaded all at once in memory. For example
+a typical pattern for loading the Excel content into a database would be :
+
+  my $iter = $valuereader->ivalues('MySheet');
+  my $sth  = $dbh->prepare("INSERT INTO MYTABLE(col1, col2, col3) VALUES (?, ?, ?)");
+  while (my $row = $iter->()) {
+    $sth->execute(@$row);
+  }
+  
+As another example, suppose a large population table, from which we want to produce a list of
+list of minor girls. This can be done with a combination of iterator operations : 
+
+  use Iterator::Simple       qw/igrep imap/;
+  use Iterator::Simple::Util qw/ireduce/;
+  my $minor_girls = ireduce {"$a, $b"}                              # successive results joined with ", "
+                    imap    {"$_->{firstname} $_->{lastname}"}      # produce a flat string from an input record with first/last name
+                    igrep   {$_->{gender} eq 'F' && $_->{age} < 18} # filter input records
+                            $valuereader->itable('Population');     # source iterator
+
 =head1 METHODS
 
 =head2 new
 
-  my $reader = Excel::ValueReader::XLSX->new(xlsx  => $filename_or_handle,
-                                             using => $backend,
-                                             %date_formatting_options);
+  my $reader = Excel::ValueReader::XLSX->new(xlsx  => $filename_or_handle, %options);
+  # .. or with syntactic sugar :
+  my $reader = Excel::ValueReader::XLSX->new($filename_or_handle, %options);
 
 The C<xlsx> argument is mandatory and points to the C<.xlsx> file to be parsed,
-or is an open filehandle.
-The C<using> argument is optional; it specifies the backend to be used for parsing;
-default is 'Regex'.
+or to an open filehandle.
 
-As syntactic sugar, a shorter form is admitted :
+Options are :
 
-  my $reader = Excel::ValueReader::XLSX->new($filename_or_handle);
+=over
+
+=item C<using>
+
+The backend to be used for parsing; default is 'Regex'.
+
+=item C<date_format>, C<time_format>, C<datetime_format>, C<date_formatter>
+
+Parameters for formatting date and time values; these are described in the L</DATE AND TIME FORMATS> section below.
+
+=back
 
-Optional parameters for formatting date and time values
-are described in the L</DATE AND TIME FORMATS> section below.
 
 
 =head2 sheet_names
@@ -365,6 +456,7 @@ are described in the L</DATE AND TIME FORMATS> section below.
   my @sheets = $reader->sheet_names;
 
 Returns the list of worksheet names, in the same order as in the Excel file.
+The first name in the list corresponds to sheet number 1.
 
 =head2 active_sheet
 
@@ -375,11 +467,31 @@ May return C<undef>.
 
 =head2 values
 
+  my ($ref, $grid) = $reader->values($sheet);
+  # or 
   my $grid = $reader->values($sheet);
 
-Returns a bidimensional array of scalars, corresponding to cell
-values in the specified worksheet. The C<$sheet> argument can be either
-a sheet name or a sheet position (starting at 1).
+Returns a pair where
+
+=over
+
+=item * 
+
+the first item is a string that describes the range of the sheet,
+in Excel A1 format (like for example C<A1:F12>
+
+=item *
+
+the second item is a bidimensional array of scalars (in other words,
+an arrayref of arrayrefs of scalars), corresponding to cell
+values in the specified worksheet. 
+
+
+=back
+
+The C<$sheet> argument can be either a sheet name or a sheet position (starting at 1).
+
+When called in scalar context, this method only returns the grid of values.
 
 Unlike the original Excel cells, positions in the grid are zero-based,
 so for example the content of cell B3 is in C<< $grid->[1][2] >>.
@@ -393,22 +505,40 @@ like this :
   my $nb_rows = @$grid;
   my $nb_cols = max map {scalar @$_} @$grid; # must import List::Util::max
 
+Alternatively, these numbers can also be obtained through the L</range_from_ref> method.
+
+
+=head2 ivalues
+
+  my ($ref, $iterator) = $reader->ivalues($sheet);
+  # or 
+  my $iterator = $reader->ivalues($sheet);
+  
+  while (my $row = $iterator->()) {
+    say join ", ", @$row;
+  }
+
+Like the L</values> method, except that it returns an iterator instead of a fully populated data grid.
+Data rows are retrieved through successive calls to the iterator.
+
+
 =head2 table_names
 
   my @table_names = $reader->table_names;
 
 Returns the list of names of tables registered in this workbook.
 
-
 =head2 table
 
   my $rows             = $reader->table(name => $table_name);  # or just : $reader->table($table_name)
   # or
   my ($columns, $rows) = $reader->table(name => $table_name);
   # or
-  my ($columns, $rows) = $reader->table(sheet => $sheet [, ref        => $range] 
-                                                        [, columns    => \@columns]
-                                                        [, no_headers => 1]
+  my ($columns, $rows) = $reader->table(sheet => $sheet [, ref          => $ref] 
+                                                        [, columns      => \@columns]
+                                                        [, no_headers   => 1]
+                                                        [, with_totals  => 1]
+                                                        [, want_records => 0]
                                        );
 
 In its simplest form, this method returns the content of an Excel table referenced by its table name
@@ -417,10 +547,13 @@ entry "Table tools / Design").
 The table name is passed either through the named argument C<name>, or positionally as unique argument
 to the method.
 
-Rows are returned as hashrefs, where keys of the hashes correspond to column names
-in the table. In scalar context, the method just returns an arrayref to the list of rows. In list
-context, the method returns a pair, where the first element is an arrayref of column names, and the
-second element is an arrayref to the list of rows.
+In list context, the method returns a pair, where the first element is
+an arrayref of column names, and the second element is an arrayref of rows.
+In scalar context, the method just returns the arrayref of rows. 
+
+Rows are normally returned as hashrefs, where keys of the hashes correspond to column names
+in the table. Under option C<< want_records => 0>>, rows are returned as arrayrefs, and it is up
+to the client to make the correspondance with column names in C<$columns>.
 
 Instead of specifying a table name, it is also possible to give a sheet name or sheet number.
 By default, this considers the whole sheet content as a single table, where column names
@@ -447,12 +580,18 @@ of being treated as a list of column names. In that case, since column names can
 be inferred from cell values in the first row, the C<columns> argument to the method
 must be present.
 
+=item with_totals
+
+For tables that have a "totals" row (turned on by a specific checkbox
+in the Excel ribbon), this row is normally not included in the result.
+To include it as a final row, pass a true value to the C<with_totals> option.
+
 =back
 
 
 =head1 AUXILIARY METHODS
 
-=head1 A1_to_num
+=head2 A1_to_num
 
   my $col_num = $reader->A1_to_num('A');    #   1
      $col_num = $reader->A1_to_num('AZ');   #  52
@@ -462,8 +601,59 @@ must be present.
 Converts a column expressed as a sequence of capital letters (in Excel's "A1" notation)
 into the corresponding numeric value.
 
+The module also has a global hash C<$Excel::ValueReader::XLSX::A1_to_num_memoized> where
+results from such conversions are memoized.
 
-=head1 formatted_date
+=head2 range_from_ref
+
+  my ($col1, $row1, $col2, $row2) = $reader->range_from_ref("C4:BB123");
+
+Returns the coordinates of the topleft and bottomright cells corresponding to a given Excel range.
+
+
+=head2 table_info
+
+  my $info = $reader->table_info->{$table_name};
+
+Returns information about an Excel table in the form of a hashref with keys
+
+=over
+
+=item name
+
+the name of the table
+
+=item ref
+
+the range of the table, in Excel notation (e.g "G6:Z44")
+
+=item columns
+
+an arrayref of column names
+
+=item id
+
+numerical id of the table
+
+=item sheet
+
+numerical id of the sheet to which the table belongs
+
+=item no_headers
+
+boolean flag corresponding to the negation of the checkbox "Headers row" in Excel.
+By default tables have a header row, both in Excel and in this module.
+
+=item has_totals
+
+boolean flag corresponding to the checkbox "Totals row" in Excel.
+By default tables have no totals row, both in Excel and in this module.
+
+
+=back
+
+
+=head2 formatted_date
 
   my $date = $reader->formatted_date($numeric_date, $excel_date_format);
 
@@ -474,6 +664,10 @@ only for for deciding if the numeric value should be presented as a date, as a t
 or both. Optionally, a custom date formatter callback could be passed as third argument.
 
 
+
+
+
+
 =head1 DATE AND TIME FORMATS
 
 =head2 Date and time handling
@@ -581,7 +775,7 @@ output is incorrect with respect to the original Excel data.
 Embedded newline characters in strings are stored in Excel as C<< \r\n >>,
 following the old Windows convention. When retrieved through the C<Regex>
 backend, the result contains the original C<< \r >> and C<< \n >> characters;
-but when retrieved through the LibXML, C<< \r >> are silently removed by the
+but when retrieved through the C<LibXML> backend, C<< \r >> are silently removed by the
 C<XML::LibXML> package.
 
 =back
@@ -594,9 +788,7 @@ L<https://www.ecma-international.org/publications/standards/Ecma-376.htm>.
 Introductory material on XLSX file structure can be found at
 L<http://officeopenxml.com/anatomyofOOXML-xlsx.php>.
 
-The CPAN module L<Data::XLSX::Parser> is claimed to be in alpha stage;
-it seems to be working but the documentation is insufficient -- I had 
-to inspect the test suite to understand how to use it.
+Concurrent modules L<Spreadsheet::ParseXLSX> or L<Data::XLSX::Parser>.
 
 Another unpublished but working module for parsing Excel files in Perl
 can be found at L<https://github.com/jmcnamara/excel-reader-xlsx>.
@@ -607,26 +799,16 @@ through the L<DateTime::Format::Excel> module.
 
 =head1 BENCHMARKS
 
-Below are some benchmarks computed with the program C<benchmark.pl> in
-this distribution. The task was to parse an Excel file of five worksheets
-with about 62600 rows in total, and report the number of rows per sheet.
-Reported figures are in seconds.
+Below are some comparative figures. The task
+computed here was to read a large Excel file with 800131 rows of 7 columns,
+and report the total number of rows. Reported figures are in seconds.
 
-  Excel::ValueReader::XLSX::Regex    11 elapsed,  10 cpu, 0 system
-  Excel::ValueReader::XLSX::LibXML   35 elapsed,  34 cpu, 0 system
-  [unpublished] Excel::Reader::XLSX  39 elapsed,  37 cpu, 0 system
-  Spreadsheet::ParseXLSX            244 elapsed, 240 cpu, 1 system
-  Data::XLSX::Parser                 37 elapsed,  35 cpu, 0 system
-
-These figures show that the regex version is about 3 times faster
-than the LibXML version, and about 22 times faster than
-L<Spreadsheet::ParseXLSX>. Tests with a bigger file of about 90000 rows
-showed similar ratios.
-
-Modules
-C<Excel::Reader::XLSX> (unpublished) and L<Data::XLSX::Parser>
-are based on L<XML::LibXML> like L<Excel::ValueReader::XLSX::Backend::LibXML>;
-execution times for those three modules are very close.
+  Spreadsheet::ParseXLSX                     1272 elapsed, 870 cpu, 4 system
+  Data::XLSX::Parser                          125 elapsed, 107 cpu, 1 system
+  Excel::ValueReader::XLSX::Regex              40 elapsed,  32 cpu, 0 system
+  Excel::ValueReader::XLSX::Regex, iterator    34 elapsed,  30 cpu, 0 system
+  Excel::ValueReader::XLSX::LibXML            101 elapsed,  83 cpu, 0 system
+  Excel::ValueReader::XLSX::LibXML, iterator   91 elapsed,  80 cpu, 0 system
 
 =head1 ACKNOWLEDGMENTS
 
@@ -645,6 +827,10 @@ Ulibuck signaled bugs several minor bugs on the LibXML backend.
 
 H.Merijn Brand suggested additions to the API and several improvements to the code source.
 
+=item *
+
+Ephraim Stevens signaled a bug in the table() method with 'ref' param.
+
 =back
 
 
@@ -654,7 +840,7 @@ Laurent Dami, E<lt>dami at cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2020-2023 by Laurent Dami.
+Copyright 2020-2025 by Laurent Dami.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
