@@ -2,12 +2,11 @@ package Excel::ValueReader::XLSX::Backend::Regex;
 use utf8;
 use 5.12.1;
 use Moose;
-use Scalar::Util qw/looks_like_number/;
-use Carp         qw/croak/;
+use Scalar::Util             qw/looks_like_number/;
+use Iterator::Simple         qw/iter/;
 
 extends 'Excel::ValueReader::XLSX::Backend';
 
-our $VERSION = '1.13';
 
 #======================================================================
 # LAZY ATTRIBUTE CONSTRUCTORS
@@ -104,12 +103,7 @@ sub _extract_xf {
 
   my @xf_nodes;
   while ($xml =~ /$xf_node_regex/g) {
-    my $all_attrs = $1;
-    my %attr;
-    while ($all_attrs =~ m[(\w+)="(.+?)"]g) {
-      $attr{$1} = $2;
-    }
-    push @xf_nodes, \%attr;
+    push @xf_nodes, _xml_attrs($1);
   }
   return @xf_nodes;
 }
@@ -119,19 +113,25 @@ sub _extract_xf {
 # METHODS
 #======================================================================
 
-sub values {
-  my ($self, $sheet) = @_;
-  my @data;
-  my ($cell_type, $seen_node);
+sub _values {
+  my ($self, $sheet, $want_iterator) = @_;
 
-  # regexes for extracting information from cell nodes
-  state $row_regex = qr(
+  # regex for the initial preamble
+  state $preamble_regex = qr(
+     <dimension\s+ref="([A-Z]+\d+(?::[A-Z]+\d+)?)"/>    # node specifying the range of defined cells
+     .*?
+     <sheetData>                                        # start container node for actual rows and cells content
+     )xs;
+
+  # regex for extracting information from cell nodes
+  state $row_or_cell_regex = qr(
      <(row)                  # row tag ($1)
        (?:\s+r="(\d+)")?     # optional row number ($2)
        [^>/]*?               # unused attrs
      >                       # end of tag
-    )x;
-  state $cell_regex = qr(
+
+     |                       # .. or ..
+
      <(c)                    # cell tag ($3)
       (?: \s+ | (?=>) )      # either a space before attrs, or end of tag
       (?:r="([A-Z]+)(\d+)")? # capture col ($4) and row ($5)
@@ -141,74 +141,105 @@ sub values {
      (?:                     # non-capturing group for an alternation :
         />                   # .. either an xml closing without content
       |                      # or
-        >                    # .. closing xml tag, followed by
+        >                    # .. closing xml tag, followed by ..
       (?:
-
          <v>(.+?)</v>        #    .. a value ($8)
         |                    #    or 
           (.+?)              #    .. some node content ($9)
        )
        </c>                  #    followed by a closing cell tag
       )
-    )x;
-  state $row_or_cell_regex = qr($row_regex|$cell_regex);
-  # NOTE : these regexes uses positional capturing groups; it would be more readable with named
-  # captures instead, but it doubles the execution time on big Excel files, so I
+    )xs;
+  # NOTE : this regex uses positional capturing groups; it would be more readable with named
+  # captures instead, but this would double the execution time on big Excel files, so I
   # stick to plain old capturing groups.
 
   # does this instance want date formatting ?
   my $has_date_formatter = $self->frontend->date_formatter;
 
-  # parse worksheet XML, gathering all cells
+  # get worksheet XML
   my $contents = $self->_zip_member_contents($self->_zip_member_name_for_sheet($sheet));
 
-  # loop on matching nodes
-  my ($row, $col) = (0, 0);
-  while ($contents =~ /$row_or_cell_regex/g) {
-    if ($1) {                # this is a 'row' tag
-      $row = $2 // $row+1;
-      $col = 0;
-    }
-    elsif ($3) {             # this is a 'c' tag
-      my ($col_A1, $given_row, $style, $cell_type, $val, $inner) = ($4, $5, $6, $7, $8, $9);
+  # parse the preamble
+  my ($ref) = $contents =~ /$preamble_regex/g; # /g to leave the pos() cursor before the 1st cell
 
-      # row and column for this cell -- either given, or incremented from last cell
-      ($col, $row) = $col_A1 && $given_row ? ($self->A1_to_num($col_A1), $given_row)
-                                           : ($col+1,                    $row);
+  # variables for the closure below
+  my ($row_num, $col_num, @rows) = (0, 0);
 
-      # handle cell value according to cell type
-      $cell_type //= '';
-      if ($cell_type eq 'inlineStr') {
-        # this is an inline string; gather all <t> nodes within the cell node
-        $val = join "", ($inner =~ m[<t>(.+?)</t>]g);
-        _decode_xml_entities($val) if $val;
+  # dual closure : may be used as an iterator or as a regular sub, depending on $want_iterator. Of course
+  # it would have been simpler to just write an iterator, and call it in a loop if the client wants all rows
+  # at once ... but thousands of additional sub calls would slow down the process. So this more complex implementation
+  # is for the sake of processing speed.
+  my $get_values = sub {
+
+    # in iterator mode, if we have a row ready, just return it
+    return shift @rows if $want_iterator and @rows > 1;
+
+    # otherwise loop on matching nodes
+    while ($contents =~ /$row_or_cell_regex/cg) { # /g allows the iterator to remember where the last cell left off
+      if ($1) {                # this is a 'row' tag
+        my $prev_row = $row_num;
+        $row_num     = $2 // $row_num+1; # if present, capture group $2 is the row number
+        $col_num     = 0;
+        push @rows, [] for 1 .. $row_num-$prev_row;
+
+        # in iterator mode, if we have a closed empty row, just return it
+        return shift @rows if $want_iterator and @rows > 1;
       }
-      elsif ($cell_type eq 's') {
-        # this is a string cell; $val is a pointer into the global array of shared strings
-        $val = $self->strings->[$val];
-      }
-      else {
-        # this is a plain value
-        ($val) = ($inner =~ m[<v>(.*?)</v>])           if !defined $val && $inner;
-        _decode_xml_entities($val) if $val && $cell_type eq 'str';
+      elsif ($3) {             # this is a 'c' tag
+        my ($col_A1, $given_row, $style, $cell_type, $val, $inner) = ($4, $5, $6, $7, $8, $9);
 
-        # if necessary, transform the numeric value into a formatted date
-        if ($has_date_formatter && $style && looks_like_number($val) && $val >= 0) {
-          my $date_style = $self->date_styles->[$style];
-          $val = $self->formatted_date($val, $date_style)    if $date_style;
+        # deal with the row number given in the 'r' attribute, if present
+        $given_row //= $row_num;
+        if    ($given_row < $row_num) {die "cell claims to be in row $given_row while current row is $row_num"}
+        elsif ($given_row > $row_num) {push @rows, [] for 1 .. $given_row-$row_num;
+                                       $col_num = 0;
+                                       $row_num = $given_row;}
+
+
+        # deal with the col number given in the 'r' attribute, if present
+        if ($col_A1) {$col_num = $Excel::ValueReader::XLSX::A1_to_num_memoized{$col_A1}
+                             //= Excel::ValueReader::XLSX->A1_to_num($col_A1)}
+        else         {$col_num++}
+
+        # handle the cell value according to cell type
+        $cell_type //= '';
+        if ($cell_type eq 'inlineStr') {
+          # this is an inline string; gather all <t> nodes within the cell node
+          $val = join "", ($inner =~ m[<t>(.+?)</t>]g);
+          _decode_xml_entities($val) if $val;
         }
+        elsif ($cell_type eq 's') {
+          # this is a string cell; $val is a pointer into the global array of shared strings
+          $val = $self->strings->[$val];
+        }
+        else {
+          # this is a plain value
+          ($val) = ($inner =~ m[<v>(.*?)</v>])           if !defined $val && $inner;
+          _decode_xml_entities($val) if $val && $cell_type eq 'str';
+
+          # if necessary, transform the numeric value into a formatted date
+          if ($has_date_formatter && $style && looks_like_number($val) && $val >= 0) {
+            my $date_style = $self->date_styles->[$style];
+            $val = $self->formatted_date($val, $date_style)    if $date_style;
+          }
+        }
+
+        # insert this value into the last row
+        $rows[-1][$col_num-1] = $val;
       }
-
-      # insert this value into the global data array
-      $data[$row-1][$col-1] = $val;
+      else {die "found a node which is neither a <row> nor a <c> (cell)"}
     }
-    else {die "unexpected regex match"}
-  }
 
-  # insert empty arrayrefs for empty rows
-  $_ //= [] foreach @data;
+    # end of regex matches. In iterator mode, return a row if we have one
+    return @rows ? shift @rows : undef if $want_iterator;
+  };
 
-  return \@data;
+  # decide what to return depending on the dual mode
+  my $retval = $want_iterator ? iter($get_values)         
+                              : do {$get_values->(); \@rows}; # run the closure and return the rows
+
+  return ($ref, $retval);
 }
 
 
@@ -223,24 +254,23 @@ sub _table_targets {
 sub _parse_table_xml {
   my ($self, $xml) = @_;
 
-  state $table_regex = qr{
-     <table .+? displayName="(\w+)"
-            .+? ref="([:A-Z0-9]+)"
-            .+? (headerRowCount="0")?
-            .+?>
-    }x;
+  $xml =~ m[<table (.*?)>]g and my $table_attrs = _xml_attrs($1)
+    or die "invalid table XML: $xml";
 
-  # extract relevant attributes from the <table> node
-  my ($name, $ref, $no_headers) = $xml =~ /$table_regex/g
-    or croak "invalid table XML";
+  # extract relevant attributes
+  my %table_info = (
+    name       => $table_attrs->{displayName},
+    ref        => $table_attrs->{ref},
+    no_headers => exists $table_attrs->{headerRowCount} && !$table_attrs->{headerRowCount},
+    has_totals => $table_attrs->{totalsRowCount},
+    columns    => [$xml =~ m{<tableColumn [^>]+? name="([^"]+)"}gx],
+   );
 
-  # column names. Other attributes from <tableColumn> nodes are ignored.
-  my @columns = ($xml =~ m{<tableColumn [^>]+? name="([^"]+)"}gx);
 
   # decode entites for all string values
-  _decode_xml_entities($_) for $name, @columns;
+  _decode_xml_entities($_) for $table_info{name}, @{$table_info{columns}};
 
-  return ($name, $ref, \@columns, $no_headers);
+  return \%table_info;
 }
 
 
@@ -264,6 +294,15 @@ sub _decode_xml_entities {
 }
 
 
+sub _xml_attrs {
+  my $attrs_list = shift;
+  my %attr       = $attrs_list =~ m[(\w+)="(.+?)"]g;
+  return \%attr;
+}
+
+
+
+
 1;
 
 __END__
